%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [tROS,SIM,tData,yData]=gly(p)
 
    % Glycolytic pathway and metabolism downstream of pyruvate in L.Lactis
    % Uses original function from Voit IEE Syst Biol 2006
    % 
 
    global ATPSpline PiSpline NADSpline
    
    % Read Data from Mat file
    load Lactococcus_Data
        
    % Time points
    tStart = 1;
    tEnd = 30;
    tData = Data(tStart:tEnd,1);
    
    %-----Concentration Values
    GlucoseData=Data(tStart:tEnd,2);
    LactateData=Data(tStart:tEnd,3);
    AcetateData=Data(tStart:tEnd,4);
    
    PEPData=Data(tStart:tEnd,6);
    P3GAData=Data(tStart:tEnd,7);
    
    FBPData=Data(tStart:tEnd,8);
    
    %modeled variables only glu,fbp,3pga,pep,lac
    yData = horzcat(GlucoseData, FBPData, P3GAData, PEPData, LactateData);
       
    %****add new********
    PGAPEPData = P3GAData + PEPData;
    K45=2.04035;  %****from measurments - Karl's estimation
    P3GAData = K45/(1+K45) * PGAPEPData;
    PEPData = 1/(1+K45) * PGAPEPData;
     
    %------Offline Concentration Values
    NADData=Data(tStart:tEnd,5);
    tData2=Data(tStart:tEnd,9); %different times for NTP and Pi measurments!
%     tStart2=1;
%     tEnd2=30;
    ATPData=Data(tStart:tEnd,10);
    PiData=Data(tStart:tEnd,11);
    
    %-------splines/smoothing
    %NADSpline = spline(tData, NADData);
    
    k=1;
    NADSpline = spline(tData(2:end), k*[0 NADData(2:end)' 0]); %exclude first point and impose zero derivates in min and max (y)
    NADSpline = fnxtr(NADSpline,1);
    
    %ATPSpline = spline(tData, ATPData);
    k=1;
    ATPSpline = spline(tData2(1:7), k*[0 0.01 ATPData(2:6)' 0.01 0]); %ATP(t0)=ATP(tf)=0.01 and impose 0 derivatives at extreme
    ATPSpline = fnxtr(ATPSpline,1);  %extrapolate values outside interval tData2(1:7) as one-order
    
    %test other approximations
%     coeff=polyfit(tData2(1:7)',[0.01 ATPData(2:6)' 0.01],3);
%     ATPSpline=mkpp([0 15.1250 80], [coeff;0 0 0 0.01]);
    
    %PiSpline = spline(tData, PiData);
    %PiSpline = spline(tData2([1 3:27]), [0 36.94 PiData(3:27)' 0]); %exclude missing value at tData(2) and Pi(2), same zero derivative
    
    k=1;
    PiSpline = spline(tData2([1 2 3:27]), k*[0 36.94 15 PiData(3:27)' 0]); %impose missing value at tData(2) - tune this, same zero derivative
    PiSpline = fnxtr(PiSpline,1);    %extrapolate maintaining derivative
    
    %save splines_AER NADSpline ATPSpline PiSpline 
 
    %------Initial Concentration Values
    X0old = yData(1,:);    %now is just 5 rows
    
    %Karl's initial conditions
    X0 = [40,  0.1,  0.72,   40,    0.1,      0.01 ];
%         Glu  G6P   FBP   PGAPEP   Pyruvate, Lactacte   
          
    %load paramsAER --> ###substitute by given - not alter names here
    params = p;
    
    
    %[0.053, 0.042, 2.6809, ...
   % 7.2032,0.9975,-1.4864,0.3858,0.3459,1.544,1.516,0.3384,1.093,0.2584,-0.0967,0.1342,0.094,0.8624,0.7663,0.0382,0.2111,0.0325,0.6755,1.0322,-0.0519,1.7474,1.4031,2.0404;]
    
    %[tROS, XROS] = ode15s(@returnVelocity, tData, X0, [], params);
    %increase resolution for better plotting
    %options=odeset('AbsTol',eps,'RelTol',eps)
    %options=odeset('AbsTol',1e-10,'RelTol',1e-10)
    %[tROS, XROS] = ode15s(@returnVelocity, linspace(tData(1),tData(end),500), X0, options,params);
    [tROS, XROS] = ode15s(@returnVelocity, linspace(tData(1),tData(end)), X0, [],params);

    % Save Results
    %save Glycolysis_estimation_and_simulation_results_vC_AER params tData yData tROS XROS
    
    %Plot_Species
    
    %real...
    %plot(tData,yData,'.', 'LineWidth',2);   %original data - plot modeled variabels glu,fbp,3pga,pep,lac
    %hold on;
    
    %and simulated...
    SIM = [XROS(:,[1 3]) K45/(1+K45)*XROS(:,4) 1/(1+K45)*XROS(:,4) XROS(:,[6])];
%     plot(tROS,SIM,'LineWidth', 2);
% %     plot(tROS, XROS(:,[1 3 6]), 'LineWidth', 2);   %simulations of Glu  G6P   FBP   PGAPEP   Pyruvate Lactacte
% %     %plot(tData,P3GAData,'gx','linewidth',2);
% %     plot(tROS,K45/(1+K45)*XROS(:,4),'g-','LineWidth',2);
% %     %plot(tData,PEPData,'cx','linewidth',2);
% %     plot(tROS,1/(1+K45)*XROS(:,4),'c-','LineWidth',2);
% %     %lactacte at the end
% %     %plot(tROS, XROS(:,[6]), 'LineWidth', 2);   %simulations of Glu  G6P   FBP   PGAPEP   Pyruvate Lactacte
% %     %legend({'Glu', 'FBP', '3PGA' 'PEP' 'Lac', 'Gluf' , 'G6Pf', 'FBPf', 'PGAPEPf', 'Pyrf', 'Lacf','3PGA','PGAf', 'PEP', 'PEPf'});
%     legend({'Glu', 'FBP', '3PGA' ,'PEP', 'Lac', 'Gluf' , 'FBPf', 'PGAf', 'PEPf', 'Lacf'});
%     xlabel('Time [min]');
%     ylabel('concentration [mM]');
%     title('Glucose pulse 40mM - Aerobic conditions');

%     %plot(tData,GlucoseData,'.-');
%     plot(tData,P3GAData,'.');
%     plot(tData,PEPData,'.');
   
 
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dX = returnVelocity(t, X, param)
    
    global NADSpline ATPSpline PiSpline
    
    sfg=1e-5;

    %input signals
    NAD = ppval(NADSpline, t); %u0   
    ATP = ppval(ATPSpline, t);  %u1
    Pi  = ppval(PiSpline, t);    %u2
 
%  %Karl's initial conditions
%     X0 = [40,  0.1,  0.72,   40,    0.1,      0.01 ];
% %         Glu  G6P   FBP   PGAPEP   Pyruvate, Lactacte   
             
    Glu = X(1);
    G6P = X(2);
    FBP = X(3);
    PGAPEP = X(4);
    
    K45 = 2.0403500; %now it's global- doesn't work
    PEP = 1/(1+K45) * PGAPEP;
    
    Pyruvate = X(5);
    Lactacte = X(6); 
   % Acetate = X(5); %needed? NO
   
    %fluxes
    vGlu        = - param(1) * (1 + param(2)*(t+sfg)^param(3)) * Glu;
    
    vG6P        = param(4)*((Glu+sfg)^param(5))*((G6P+sfg)^param(6))*((PEP+sfg)^param(7))  - param(8)*((G6P+sfg)^param(9))*(ATP^param(10));
    
    vFBP        = param(8)*((G6P+sfg)^param(9))*(ATP^param(10))                - param(11)*((FBP+sfg)^param(12))*(Pi^param(13))*(NAD^param(14));
    
    %merge signals 3PGA+PEP------------

    vPGAPEP = 2*param(11)*((FBP+sfg)^param(12))*(Pi^param(13))*(NAD^param(14)) - param(4)*((Glu+sfg)^param(5))*((G6P+sfg)^param(6))*((PEP+sfg)^param(7)) ...            
                                                                         - param(15)*((PEP+sfg)^param(16)) ...
                                                                         - param(17)*((FBP+sfg)^param(18))*((PEP+sfg)^param(19))*(Pi^param(20));                                                             
    %----------------------------------   
    
    vPyruvate   = param(4)*((Glu+sfg)^param(5))*((G6P+sfg)^param(6))*((PEP+sfg)^param(7)) ...
                  + param(17)*((FBP+sfg)^param(18))*((PEP+sfg)^param(19))*(Pi^param(20))    - param(21)*((Pyruvate+sfg)^param(22))*((FBP+sfg)^param(23))*(NAD^param(24)) ...
                                                                                - param(25)*((Pyruvate+sfg)^param(26));
 
    vLactate    = param(21)*((Pyruvate+sfg)^param(22))*((FBP+sfg)^param(23))*(NAD^param(24));

    dX = [vGlu; vG6P; vFBP; vPGAPEP; vPyruvate; vLactate];
         % Glu  G6P   FBP   PGAPEP   Pyruvate, Lactacte   
    
    dX = real(dX);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
